# By hads <hads@nice.net.nz>
# Released under the MIT license

import os
import sys
import subprocess
import signal
import re

from base import *

class Snooper(object):
    # regex's for packet data extraction
    decode_regex = re.compile(r'\[= (.*?)\]$')
    detail_regex = re.compile(r'[char|name]: "(.*?)"  -- Charset')
    
    programs = []
    unique = {}

    # Maximum number of packets with no data before we stop
    nilpkts = 2500
    
    # Key counters
    events = 0
    packets = 0

    def __init__(self, adapter, quiet=False):
        self.quiet = quiet
        self.adapter = adapter

    def processPacket(self, pkt):
        found = 0

        # Process the packet
        self.packets += 1
        event_id = None
        for data in pkt:
            # Channel ID for this packet
            if data[:10] == "Service_ID":
                channel = data.split(': ')[1:][0].split()[0]

            # Are we in en event
            if data[:8] == "Event_ID":
                # Store old event and create a new one
                if event_id:
                    key = channel + "|" + event_id
                    if not self.unique.has_key(key):
                        found += 1
                        self.unique[key] = True
                        self.programs.append(event)
                event_id = data.split(': ')[1:][0].split()[0]
                event = Program()
                event['pid'] = channel
                self.events += 1
                continue

            # End of packet store last event
            if data[:3] == "CRC":
                if event_id:
                    key = channel + "|" + event_id
                    if not self.unique.has_key(key):
                        found += 1
                        self.unique[key] = True
                        self.programs.append(event)

            # Check for event data
            if event_id:
                # Starttime
                if data[:11] == "Start_time:":
                    event['start'] = ' '.join(data.split(': ')[1:]).split()[0]
                    event['startinfo'] = self.decode_regex.findall(data)[0].strip()
                    continue
                # Duration
                if data[:9] == "Duration:":
                    event['duration'] = ' '.join(data.split(': ')[1:]).split()[0]
                    event['durationinfo'] = self.decode_regex.findall(data)[0].strip()
                    continue
                # Name
                if data[:11] == "event_name:":
                    event['title'] = self.detail_regex.findall(data)[0].decode('latin-1')
                    continue
                # Description
                if data[:10] == "text_char:":
                    try:
                        event['description'] = self.detail_regex.findall(data)[0].decode('latin-1')
                    except:
                        event['description'] = ""
                    continue
                # Rating
                if data[:7] == "Rating:":
                    event['ratingnum'] = ' '.join(data.split(': ')[1:]).split()[0]
                    event['ratinginfo'] = self.decode_regex.findall(data)[0].strip()
                    continue
                # Country
                if data[:13] == "Country_code:":
                    event['country'] = ' '.join(data.split(': ')[1:]).strip()
                    continue
                # Language
                if data.find("language_code:") >= 0:
                    event['language'] = ' '.join(data.split(': ')[1:]).strip()
                    continue
                # Content
                if data[:23] == "Content_nibble_level_1:":
                    event['content_1'] = ' '.join(data.split(': ')[1:]).split()[0]
                    continue
                if data[:23] == "Content_nibble_level_2:":
                    event['content_2'] = ' '.join(data.split(': ')[1:]).split()[0]
                    continue
                if data[:3] == "[= ":
                    event['contentinfo'] = self.decode_regex.findall(data)[0].strip()
                    continue
                # User
                if data[:14] == "User_nibble_1:":
                    event['user_1'] = ' '.join(data.split(': ')[1:]).split()[0]
                    continue
                if data[:14] == "User_nibble_2:":
                    event['user_2'] = ' '.join(data.split(': ')[1:]).split()[0]
                    continue
        # Found how many shows?
        return found

    def snoop(self):
        # Open stream
        command = ('dvbsnoop', '-adapter', self.adapter, '-nph', '0x12')

        self.snoop = subprocess.Popen(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            close_fds=True)

        self.snoop.stderr.close()

        # Loop packets
        check = i = 0
        if not self.quiet:
            s = StatusDisplay()
            sys.stderr.write('\n')
        while check < self.nilpkts:
            i = i + 1
            if not self.quiet:
                s.out('Processing packets: %05d' % i)
        
            # Get packet start
            out = self.snoop.stdout.readline()
            while out and out[:11] != "SECT-Packet":
                out = self.snoop.stdout.readline()
        
            # Packetize
            pkt = []
            pkt.append(out.strip())
            while out and out[:3] != "CRC":
                out = self.snoop.stdout.readline()
                pkt.append(out.strip())
        
            # Process the packet
            found = self.processPacket(pkt)
            if found > 0:
                check = 0
            else:
                check += 1
        
            # Had enoungh
            if check >= self.nilpkts:
                break
        
        # Natural completion ... kill snoop
        self.kill()
        return self.programs

    def kill(self):
        os.kill(self.snoop.pid, signal.SIGTERM)




# By hads <hads@nice.net.nz>
# Released under the MIT license

import os
import re
import copy
import logging
import ConfigParser
from urllib import urlopen

from base import *

log = logging.getLogger(NAME)

class BaseProcessor(object):
    valid = True

    def __init__(self, config):
        self.config = config

    def __call__(self, programs):
        self.programs = programs
        if self.valid:
            log.info('Processing programs with %s processor' % self.__class__.__name__)
            for program in self.programs:
                if program.isValid():
                    self.process(program)
                else:
                    if 'title' in program:
                        log.debug(
                            'Ignoring invalid program (PID:%s) (Title:%s)', program['pid'], program['title']
                        )
                        if 'start' not in program:
                            log.debug('No start')
                        if 'end' not in program:
                            log.debug('No end')
                        if 'channel' not in program:
                            log.debug('No channel')
                    else:
                        log.debug('Ignoring invalid program (no title)')
            self.postProcess()
        return self.programs

    def process(self, program):
        raise NotImplementedError

    def postProcess(self):
        pass

class StripHtml(BaseProcessor):
    def process(self, program):
        program['title'] = re.sub('<.*?>', '', program['title'])

class HD(BaseProcessor):
    regex = re.compile(r'HD$')
    
    def process(self, program):
        matched = self.regex.search(program['description'])
        if matched:
            log.debug("Found HD program: '%s'", program['title'])
            program['video'] = True
            program['hd'] = True
            # I imagine all HD programs will be widescreen
            program['aspect'] = '16:9'
            program['description'] = self.regex.sub('', program['description'])

class Widescreen(BaseProcessor):
    regex = re.compile(r' \(WS\)')
    
    def process(self, program):
        matched = self.regex.search(program['description'])
        if matched:
            log.debug("Found widescreen program: '%s'", program['title'])
            program['video'] = True
            program['aspect'] = '16:9'
            program['description'] = self.regex.sub('', program['description'])

class Credits(BaseProcessor):
    actor_regex = re.compile(r'\. Starring: (.*?)\.')
    director_regex = re.compile(r"Directed by (([A-Za-z'\-]+(\s|.))+)")
    
    def process(self, program):
        matched = self.actor_regex.search(program['description'])
        if matched:
            log.debug("Found actor in program: '%s'", program['title'])
            program['actors'] = matched.group(1).split(', ')
            #program['description'] = self.regex.sub('', program['description'])
        matched = self.director_regex.search(program['description'])
        if matched:
            log.debug("Found director in program: '%s'", program['title'])
            program['director'] = matched.group(1).strip(' .')
            #program['description'] = self.regex.sub('', program['description'])

class Year(BaseProcessor):
    regex = re.compile(r' \((\d{4})\)\.$')
    
    def process(self, program):
        matched = self.regex.search(program['description'])
        if matched:
            log.debug("Found year %s for program: '%s'", matched.groups()[0], program['title'])
            program['year'] = matched.groups(1)[0]
            #program['description'] = self.regex.sub('', program['description'])

class MovieTitle(BaseProcessor):
    regex = re.compile(
        r'''(
            Movie|
            Saturday\sBlockbuster|
            Blockbuster\sTuesday|
            Sunday\sPremiere\sMovie|
            Sunday\sBlockbuster\sPremiere|
            Sunday\sPremier\sMovie|
            Mid-Week\sMovi?e|
            The\sSol\sSunday\sNight\sMovie|
            Saturday\sComedy\sBlockbuster
            ):\s?''',
        re.VERBOSE
    )
    
    def process(self, program):
        matched = self.regex.match(program['title'])
        if matched:
            log.debug("Found movie from title: '%s'", program['title'])
            program['category_type'] = 'movie'
            program['title'] = self.regex.sub('', program['title'])

class Subtitle(BaseProcessor):
    regexes = (
        re.compile(r"(Today|Tonight)?:? ?'(?P<subtitle>.*?)'\.\s?"),
        re.compile(r"'(?P<subtitle>.{2,60}?)'\s"),
        re.compile(r"(?P<subtitle>.{2,60}?):\s"),
    )

    def process(self, program):
        if 'description' in program:
            for regex in self.regexes:
                matched = regex.match(program['description'])
                if matched:
                    program['category_type'] = 'series'
                    log.debug('Found subtitle in %s', program['description'])
                    program['subtitle'] = matched.group('subtitle')
                    program['description'] = regex.sub('', program['description'])

class MovieDesc(BaseProcessor):
    regex = re.compile(
        r'''(
            Action|
            Adventure|
            Animated|
            Comedy|
            Crime|
            Documentary|
            Drama|
            Family|
            Horror|
            Magazine|
            Musical|
            Romantic\sComedy|
            Rom\sCom|
            Thriller|
            Biography/Drama
        )(?:, (\d{4}))?:\s?''',
        re.VERBOSE
    )
    
    def process(self, program):
        if 'description' in program:
            matched = self.regex.match(program['description'])
            if matched:
                log.debug("Found movie from description in '%s'", program['title'])
                program['category_name'] = matched.groups()[0]
                if matched.groups()[1]:
                    program['year'] = matched.groups()[1]
                program['category_type'] = 'movie'
                program['description'] = self.regex.sub('', program['description'])

class CategoryList(BaseProcessor):
    CATEGORY_TYPES = {
        '0': 'tvshow',
        '1': 'movie',
        '2': 'tvshow',
        '3': 'tvshow',
        '4': 'sports',
        '5': 'tvshow',
        '6': 'tvshow',
        '7': 'tvshow',
        '8': 'tvshow',
        '9': 'tvshow',
        '10': 'tvshow',
        '11': 'tvshow',
        '15': 'tvshow',
    }
    CATEGORIES = {
        # Special
        #'0-0': '',
        #'0-11': '',

        # Movies
        '1-0': 'Drama',
        '1-1': 'Thriller/Crime',
        '1-2': 'Action/Adventure',
        '1-3': 'Thriller/Crime',
        '1-4': 'Comedy',
        '1-5': 'Drama',
        '1-6': 'Family/Romance',
        '1-7': 'Classical/Religious/Historical',
        '1-8': 'Adult',
        '1-9': 'Religious',
        '1-10': 'Thriller',
        '1-12': 'News/Magazine',
        '1-13': 'War',
        '1-14': 'Western',
        '1-15': 'Making of',

        # News/Documentary
        '2-0': 'News/Current Affairs',
        '2-1': 'News',
        '2-2': 'Magazine',
        '2-3': 'Documentary',
        '2-4': 'Discussion/Interview',
        '2-15': 'Sports',

        # General shows
        '3-0': 'General Show',
        '3-1': 'Game Show/Quiz/Contest',
        '3-2': 'Variety Show',
        '3-3': 'Talk Show',
        '3-4': 'Reality',
        '3-5': 'Reality/Stunt',
        '3-6': 'Drama',
        '3-7': 'Reality',
        '3-8': 'Reality',
        '3-10': 'Science Fiction',
        '3-11': 'Crime',
        '3-12': 'Sports - Wrestling/Fighting',
        '3-13': 'Special Event',
        '3-14': 'Adult',

        # Sports
        '4-0': 'Sports',
        '4-1': 'Sports - Special Event',
        '4-2': 'Sports - Golf', # Magazine?
        '4-3': 'Sports - Soccer',
        '4-4': 'Sports - Tennis/Squash',
        '4-5': 'Team Sports',
        '4-6': 'Sports - Athletics',
        '4-7': 'Motor Sports',
        '4-8': 'Water Sports',
        '4-9': 'Winter Sports',
        '4-10': 'Sports - Equestrian',
        '4-11': 'Sports - Martial Arts',
        '4-13': 'Sports - Cricket',
        '4-14': 'Sports - Cycling',
        '4-15': 'Sports Talk Show',

        # Childrens show
        '5-0': 'Childrens - General',
        '5-1': 'Childrens - Pre-school',
        '5-2': 'Childrens - Ages 6-14',
        '5-3': 'Childrens - Ages 10-16',
        '5-4': 'Childrens - Educational/Informational',
        '5-5': 'Childrens - Cartoon/Puppets',

        # Music
        '6-0': 'Music - General',
        '6-1': 'Music - Rock/Pop',
        '6-2': 'Music - Classical/Opera',
        '6-3': 'Music - Folk/Traditional',
        '6-4': 'Music - Jazz',
        '6-5': 'Music - Musical/Opera',
        '6-6': 'Music - Ballet',
        '6-7': 'Music - Religious',
        '6-8': 'Music - Countdown',
        '6-9': 'Music - Gospel',
        '6-15': 'Music - Special',

        # Arts/Culture
        '7-0': 'Arts/Culture',
        '7-1': 'Performing Arts',
        '7-2': 'Fine Arts',
        '7-3': 'Religion',
        '7-5': 'Literature',
        '7-6': 'Film/Cinema',
        '7-10': 'Magazine',

        # Social/Political/Economics
        '8-0': 'News/Current Affairs',
        '8-1': 'Magazone/Documentary',
        '8-2': 'Social Advisory',
        '8-3': 'Documentary',
        '8-4': 'Social/Political/Economics',

        # Education/Science/Factual
        '9-0': 'Education/Science/Factual',
        '9-1': 'Nature/Animals/Environment',
        '9-2': 'Technology/Natural Science',
        '9-3': 'Medicine/Physiology/Psychology',
        '9-6': 'Further Education',
        '9-8': 'Education/Science/Factual',
        '9-9': 'Education/Science/Factual',
        '9-10': 'Education/Science/Factual',
        '9-11': 'Travel',
        '9-12': 'Education/Science/Factual',
        '9-13': 'Education/Science/Factual',
        '9-14': 'Education/Science/Factual',
        '9-15': 'Education/Science/Factual',

        # Leisure/Hobbies
        '10-0': 'Leisure/Hobbies',
        '10-1': 'Tourisim/Travel',
        '10-2': 'Craft',
        '10-3': 'Fishing/Motoring',
        '10-4': 'Fitness/Health',
        '10-5': 'Cooking',
        '10-6': 'Shopping/Advertisment',
        '10-7': 'Home/Gardening',
        '10-8': 'Leisure/Hobbies',
        '10-10': 'Reality',
        '10-11': 'Home/Design',
        '10-12': 'Property/Reality',

        # Special News/Entertainment
        '11-0': 'Special News/Entertainment',
        '11-1': 'Special News/Entertainment',
        '11-3': 'Live broadcast',
        '11-5': 'Special News/Entertainment',
        '11-6': 'International',
        '11-7': 'Special News/Entertainment',
        '11-8': 'Special News/Entertainment',
        '11-9': 'Special News/Entertainment',
        '11-10': 'Special News/Entertainment',
        '11-11': 'Special News/Entertainment',
        '11-12': 'Special News/Entertainment',
        '11-13': 'Special News/Entertainment',
        '11-14': 'Special News/Entertainment',

        # Adult
        '15-0': 'Adult',
        '15-1': 'Adult',
        '15-5': 'Adult',
        '15-8': 'Adult',
    }

    def process(self, program):
        if 'content_1' in program and 'content_2' in program:
            cat_num = '%s-%s' % (program['content_1'], program['content_2'])
            try:
                program['category_type'] = self.CATEGORY_TYPES[program['content_1']]
            except KeyError:
                pass
            try:
                program['category_name'] = self.CATEGORIES[cat_num]
            except KeyError:
                pass

class CategoryDb(BaseProcessor):
    def __init__(self, config):
        BaseProcessor.__init__(self, config)
        try:
            try:
                # Try sqlite from the standard library (> 2.5)
                from sqlite3 import dbapi2 as sqlite
            except ImportError:
                # Try sqlite from the external package (< 2.4)
                from pysqlite2 import dbapi2 as sqlite
        except ImportError:
            self.valid = False
            log.info('Not using CategoryDb processor - sqlite not found.')
        else:
            try:
                database = os.path.expanduser(config.get('CategoryDb', 'database'))
            except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
                self.valid = False
                log.info('Not using CategoryDb processor - no config found.')
            else:
                self.db = sqlite.connect(database)
                self.c = self.db.cursor()
                self.c.execute("""CREATE TABLE IF NOT EXISTS categories(
                    id INTEGER PRIMARY KEY,
                    title VARCHAR,
                    cat_type VARCHAR,
                    cat VARCHAR
                    )"""
                )
    
    def process(self, program):
        self.c.execute(
            "SELECT cat_type, cat FROM categories WHERE title LIKE ?",
            (program['title'],)
        )
        row = self.c.fetchone()
        if row:
            try:
                (program['category_type'], program['category_name']) = row
                log.debug('Set category %s for %s', program['category_name'], program['title'])
            except Exception, e:
                log.debug('Exception getting category from DB: %s', e)

class SkyRatings(BaseProcessor):
    RATING_SYSTEM = 'SKY-NZ'
    RATINGS = {
        #'1': '?' # Freeview?
        '2': 'G', # Some G
        '4': 'PG', # Some PG
        '6': 'M',
        '8': 'R16',
        '10': '18+',
        '12': 'R18',
        '13': 'R20',
    }
    RATING_ADVISORIES = {
        '1': 'V',
        '2': 'S',
        '3': 'VS',
        '4': 'L',
        '5': 'VL',
        '6': 'LS',
        '7': 'VLS', # Seems to be V in SKY
        '8': 'C',
        #'9': '?', # Unknown
        #'10': '?', # Unknown
        #'13': '?', # Unknown
        #'14': '?', # Unknown
    }
    def process(self, program):
        program['rating_system'] = self.RATING_SYSTEM
        if 'ratingnum' in program:
            try:
                program['rating'] = self.RATINGS[program['ratingnum']]
            except:
                pass
        if 'user_2' in program and program['user_2'] != '0':
            try:
                program['rating_advisory'] = self.RATING_ADVISORIES[program['user_2']]
            except:
                pass


class Imdb(BaseProcessor):
    cache = {}
    
    def __init__(self, config):
        BaseProcessor.__init__(self, config)
        try:
            import imdb
        except ImportError:
            self.valid = False
            log.info('Not using ImdbProcessor - imdbpy not found.')
        else:
            try:
                access_system = config.get('Imdb', 'access_system')
                database_uri = config.get('Imdb', 'database_uri')
                self.imdb = imdb.IMDb(access_system, uri=database_uri)
            except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
                self.valid = False
                log.info('Not using ImdbProcessor - no config found.')

    def stripTitle(self, string):
        for word in ['The', 'An', 'A']:
            if string.startswith(word + ' '):
                string = string[len(word + ' '):]
            if string.endswith(', ' + word):
                string = string[:-len(word + ', ')]
        return string
    
    def process(self, program):
        imdb_match = None
        if ('category_type' in program and program['category_type'] == 'movie') or ('subtitle' in program):
            log.debug("Searching IMDB for '%s'" % program['title'])
            if program['title'] in self.cache:
                log.debug('Cache hit')
                imdb_results = self.cache[program['title']]
            else:
                log.debug('Cache miss')
                imdb_results = self.imdb.search_movie(program['title'])
                self.cache[program['title']] = imdb_results
            # Are we looking for a movie?
            if program['category_type'] == 'movie':
                for result in imdb_results:
                    if result['kind'] == 'movie':
                        # Check titles for match
                        if self.stripTitle(program['title']).lower() == self.stripTitle(result['title']).lower():
                            # TODO: Check if we've already updated
                            self.imdb.update(result)
                            if self.stripTitle(program['title']).lower() == self.stripTitle(result['title']).lower():
                                try:
                                    imdb_id = self.imdb.get_imdbID(result)
                                except:
                                    # TODO: log and narrow exception
                                    pass
                                else:
                                    imdb_match = result
                                    break
                            else:
                                continue
            # Or an episode of a program?
            elif program['subtitle']:
                for result in imdb_results:
                    if result['kind'] == 'tv series':
                        # Check titles for match
                        if self.stripTitle(program['title']).lower() == self.stripTitle(result['title']).lower():
                            # TODO: Check if we've already updated
                            self.imdb.update(result)
                            if self.stripTitle(program['title']).lower() == self.stripTitle(result['title']).lower():
                                try:
                                    imdb_id = self.imdb.get_imdbID(result)
                                except:
                                    # TODO log and narrow exception
                                    pass
                                else:
                                    imdb_match = result
                                break
                            else:
                                continue
                if imdb_match and 'episodes' in imdb_match.keys():
                    for s in imdb_match['episodes']:
                        for e in imdb_match['episodes'][s]:
                            try:
                                if imdb_match['episodes'][s][e]['episode title'].lower().strip() == program['subtitle'].lower().strip():
                                    try:
                                        program['series'] = int(s)
                                        program['episode'] = int(e)
                                    except:
                                        pass
                                    try:
                                        program['year'] = imdb_match['episodes'][s][e]['year']
                                    except KeyError:
                                        pass
                            except KeyError:
                                pass
        if imdb_match:
            log.debug('Found match for "%s" (http://imdb.com/title/tt%s/)' % (program['title'], imdb_id))
            if 'year' in imdb_match.keys():
                program['year'] = imdb_match['year']
            if 'rating' in imdb_match.keys():
                program['star_rating'] = imdb_match['rating']
            if 'runtimes' in imdb_match.keys():
                for r in imdb_match['runtimes']:
                    try:
                        program['runtime'] = int(r)
                        break
                    except:
                        pass

class SearchReplaceTitle(BaseProcessor):
    def __init__(self, config):
        BaseProcessor.__init__(self, config)
        try:
            import simplejson
        except ImportError:
            self.valid = False
            log.info('Not using SearchReplaceTitle processor - simplejson not found.')
        else:
            try:
                data = urlopen(config.get('SearchReplaceTitle', 'url')).read()
            except IOError:
                self.valid = False
                log.info('Not using SearchReplaceTitle - fetching data failed.')
            else:
                try:
                    self.replacements = simplejson.loads(data)
                except ValueError:
                    self.valid = False
                    log.info('Not using SearchReplaceTitle - JSON parse failed.')

    def process(self, program):
        for r in self.replacements:
            old_title = program['title']
            program['title'] = re.sub(r['search'], r['replace'], program['title'])
            if old_title != program['title']:
                log.debug('SearchReplaceTitle: Changed title from "%s" to "%s"', program['title'], r['replace'])

class BBCWorldOnTV1(BaseProcessor):
    programs_to_delete = []
    programs_to_insert = []

    def process(self, program):
        if program['channel'].xmltvid == 'tv1.sky.co.nz' and re.match(r'BBC World( \d{4})?', program['title']):
            for op in self.programs:
                if op.isValid() and op['channel'].xmltvid == 'bbc-world.sky.co.nz'\
                and op['start'] > program['start'] and op['end'] < program['end']:
                    np = copy.deepcopy(op)
                    np['channel'] = program['channel']
                    self.programs_to_insert.append(np)
            self.programs_to_delete.append(program)

    def postProcess(self):
        for program in self.programs_to_delete:
            log.debug('Removing program %s', program)
            self.programs.remove(program)
        for program in self.programs_to_insert:
            log.debug('Inserting program %s', program)
            self.programs.append(program)




# By hads <hads@nice.net.nz>
# Released under the MIT license

import os
import signal
import subprocess
from time import sleep

from base import *

class TuningFailed(Exception):
    pass

class Tuner(object):
    def __init__(self, adapter, lnb_offset):
        self.adapter = adapter
        self.lnb_offset = int(lnb_offset)

    def tune(self, frequency, flush=True):
        log.info('Tuning DVB card %s', self.adapter)
        freq = str((int(frequency) - self.lnb_offset) * 1000)
        self.tuner = subprocess.Popen(
            ['dvbtune', '-c', self.adapter, '-f', freq, '-s', '22500', '-p', 'h', '-m', '-tone', '0'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        sleep(0.5) # Give dvbtune time to see if the card is available
        if self.tuner.poll() is not None:
            return False

        log.debug('dvbtune process running')
        if flush:
            self.flush()
        return True

    def free(self):
        # self.tuner.kill() was only introduced in 2.6
        os.kill(self.tuner.pid, signal.SIGTERM)

    def flush(self):
        """
        It seems that dvbsnoop can output data buffered from the previous tune.
        This makes sure we discard that data, 2000 is an arbitrary number.
        """
        log.info('Flushing stale EIT data...')
        self.dvbsnoop_flush = subprocess.Popen(
            ['dvbsnoop', '-adapter', self.adapter, '-n', '2000', '-nph', '0x12'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        (stdoutdata, stderrdata) = self.dvbsnoop_flush.communicate()
        log.info('Done.')




# By hads <hads@nice.net.nz>
# Released under the MIT license

import os
import logging
import ConfigParser

from datetime import datetime, timedelta
from cgi import escape

from base import *

log = logging.getLogger(NAME)

class BaseOutputter(object):
    def __init__(self, config):
        self.config = config
    
    def __call__(self, channels, programs):
        self.channels = channels
        channels_seen = []
        output = []
        
        for program in programs:
            if program.isValid() and program['channel'].pid not in channels_seen:
                channels_seen.append(program['channel'].pid)
        
        output.append(self.header())
        for channel in channels.values():
            if channel.pid in channels_seen:
                output.append(self.channel(channel))
        for program in programs:
            if program.isValid():
                output.append(self.program(program))
        output.append(self.footer())

        output = [o for o in output if o]
        return '\n'.join(output)

    def header(self):
        pass

    def footer(self):
        pass

    def channel(self, channel):
        pass
    
    def program(self, program):
        pass

class Test(BaseOutputter):
    def channel(self, channel):
        return '%s - %s' % (channel.pid, channel.xmltvid)
    
    def program(self, program):
        return '%(title)s - %(start)s (%(duration)s)' % program

class XMLTV(BaseOutputter):
    def __init__(self, config, old_channel_ids=False):
        BaseOutputter.__init__(self, config)
        self.old_channel_ids = old_channel_ids
    
    def header(self):
        gendate = datetime.now().strftime("%Y%m%d%H%M%S %z")
        return '<?xml version="1.0" encoding="ISO-8859-1"?>\n'\
            '<!DOCTYPE tv SYSTEM "xmltv.dtd">\n'\
            '<tv generator-info-name="%s/%s" generator-info-url="%s" date="%s">' % (NAME, VERSION, URL, gendate)
    
    def footer(self):
        return '</tv>\n'
    
    def channel(self, channel):
        output = []
        try:
            show_icons = self.config.getboolean('XMLTV', 'show_icons')
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError, ValueError):
            show_icons = True
        try:
            icon_url_base = self.config.get('XMLTV', 'icon_url_base')
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
            show_icons = False
        if self.old_channel_ids:
            output.append('<channel id="%s.dvb.guide">' % channel.pid)
        else:
            output.append('<channel id="%s">' % channel.xmltvid)
        output.append('\t<display-name>%s</display-name>' % channel.name)
        if channel.icon and show_icons:
            output.append('\t<icon src="%s%s" />' % (icon_url_base, channel.icon))
        if channel.url:
            output.append('\t<url>%s</url>' % channel.url)
        output.append('</channel>')
        return '\n'.join(output)
    
    def program(self, program):
        output = []
        start = program['start'].astimezone(local)
        end = program['end'].astimezone(local)
        
        if self.old_channel_ids:
            output.append(
                '<programme channel="%s.dvb.guide" start="%s" stop="%s">' \
                % (program['channel'].pid, start.strftime("%Y%m%d%H%M%S %z"), end.strftime("%Y%m%d%H%M%S %z"))
            )
        else:
            output.append(
                '<programme channel="%s" start="%s" stop="%s">' \
                % (program['channel'].xmltvid, start.strftime("%Y%m%d%H%M%S %z"), end.strftime("%Y%m%d%H%M%S %z"))
            )
        if 'language' in program:
            output.append('\t<title lang="%s">%s</title>' % (program['language'], escape(program['title'])))
        else:
            output.append('\t<title>%s</title>' % escape(program['title']))

        if 'subtitle' in program:
            output.append('\t<sub-title>%s</sub-title>' % escape(program['subtitle']))

        if 'description' in program and program['description']:
            output.append('\t<desc>%s</desc>' % escape(program['description']))

        if 'actors' in program or 'director' in program:
            output.append('\t<credits>')
            if 'director' in program:
                output.append('\t\t<director>%s</director>' % escape(program['director']))
            if 'actors' in program:
                for actor in program['actors']:
                    output.append('\t\t<actor>%s</actor>' % escape(actor))
            output.append('\t</credits>')

        if 'year' in program:
            output.append('\t<date>%s</date>' % program['year'])
        
        if 'category_type' in program:
            output.append('\t<category>%s</category>' % program['category_type'])

        if 'category_name' in program:
            output.append('\t<category>%s</category>' % program['category_name'])

        if 'runtime' in program:
            output.append('\t<length units="minutes">%s</length>' % program['runtime'])

        if 'imdb_id' in program:
            output.append('\t<url>http://imdb.com/title/tt%s/</url>' % program['imdb_id'])
        
        if 'series'  in program and 'episode' in program:
            output.append('\t<episode-num system="xmltv_ns">%s.%s.0/1</episode-num>' % (program['series'] - 1, program['episode'] -1))

        if 'video' in program and program['video']:
            output.append('\t<video>')
            output.append('\t\t<present>yes</present>')
            if 'aspect' in program:
                output.append('\t\t<aspect>%s</aspect>' % program['aspect'])
            if 'hd' in program:
                output.append('\t\t<quality>HDTV</quality>')
            output.append('\t</video>')

        # Rating
        if 'rating' in program:
            if 'rating_advisory' in program:
                output.append('\t<rating system="%s">\n\t\t<value>%s %s</value>\n\t</rating>' \
                        % (program['rating_system'], program['rating'], program['rating_advisory']))
            else:
                output.append('\t<rating system="%s">\n\t\t<value>%s</value>\n\t</rating>' 
                        % (program['rating_system'], program['rating']))

        if 'star_rating' in program:
            output.append('\t<star-rating>\n\t\t<value>%s/10</value>\n\t</star-rating>' % program['star_rating'])

        # Close tag
        output.append('</programme>')

        output = [o.encode('latin-1','replace') for o in output]
        return '\n'.join(output)

class OldXMLTV(XMLTV):
    def __init__(self, config):
        XMLTV.__init__(self, config, old_channel_ids=True)



#!/usr/bin/python
"""
epgsnoop
========

epgsnoop is a python package which wraps the dvbsnoop program
and generates XMLTV data. The data is usable for MythTV and
other PVR systems which support the XMLTV data format.

epgsnoop is only currently known to be in use in New Zealand.
Many of the processors are possibly currently targetted to the
EIT data from there. Corrections and additions are welcome.
"""
from distutils.core import setup

setup(
    name='epgsnoop',
    version='0.83',
    description='A wrapper around the dvbsnoop program to create XMLTV comliant data.',
    long_description=__doc__,
    author='Hadley Rich',
    author_email='hads@nice.net.nz',
    url='http://nice.net.nz/epgsnoop',
    download_url='http://nice.net.nz/epgsnoop',
    packages=['epgsnoop'],
    scripts=['scripts/epgsnoop'],
    license='MIT',
    platforms='Linux',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',    
        'Development Status :: 3 - Alpha',
    ],
)



# By hads <hads@nice.net.nz>
# Released under the MIT license

# Timezone classes taken and modified from
# http://docs.python.org/lib/datetime-tzinfo.html

import os
import sys
import time
import logging
import re

from datetime import tzinfo, datetime, timedelta

NAME = 'epgsnoop'
VERSION = '0.83'
URL = 'http://nice.net.nz/epgsnoop'

log = logging.getLogger(NAME)
logging.basicConfig(level=logging.INFO, format='%(message)s')

class StatusDisplay(object):
    def __init__(self):
        self.length = 0

    def out(self, text):
        i = 0
        pre = ''
        while i < self.length:
            pre = pre + '\b'
            i = i + 1
        self.length = len(text)
        sys.stderr.write(pre + text)

class Channel(object):
    xmltvid = None
    name = None
    icon = None
    url = None
    
    def __init__(self, pid):
        self.pid = pid

class Program(dict):
    def __str__(self):
        if 'title' in self and self['title']:
            return 'Program: %s' % self['title']
        else:
            return 'Program instance'

    def __repr__(self):
        if 'title' in self:
            return 'Program: %s' % self['title']
        else:
            return 'Program instance'

    def __setitem__(self, name, value):
        if name in ('start', 'duration') and not (isinstance(value, datetime) or isinstance(value, timedelta)):
            # Convert the raw value to datetime or timedelta
            try:
                value = self.mjdToDate(value)
            except Exception, e:
                log.debug('\nError converting %s value: %s', name, e)
                return
            else:
                if name == 'duration' and 'start' in self:
                    # If we have a start and we just got duration
                    # calculate the end time
                    self['end'] = self['start'] + value
                if name == 'start' and 'duration' in self:
                    # If we have a duration and we just got start
                    # calculate the end time
                    self['end'] = value + self['duration']
            
        dict.__setitem__(self, name, value)

    def isValid(self):
        if 'title' in self and self['title'] and 'start' in self and 'end' in self and 'channel' in self:
            return True
        else:
            return False

    # Convert DVB dates, ETSI EN 300 468 (DVB SI), Annex C
    def mjdToDate(self, dvb):
        # bcd hour/min/sec
        hour = int(dvb[-6:-4])
        minute = int(dvb[-4:-2])
        second = int(dvb[-2:])

        # Date or duration
        mjd = int(dvb[:-6], 16)
        if mjd == 0:
            return timedelta(hours=hour, minutes=minute, seconds=second)

        # Intermediate calcs
        y = int((mjd - 15078.2) / 365.25)
        m = int((mjd - 14956.1 - int(y * 365.25)) / 30.6001)
        if m == 14 or m == 15:
            k = 1
        else:
            k = 0

        # Date
        m_year = y + k + 1900
        m_month = m - 1 - k * 12
        m_day = mjd - 14956 - int(y * 365.25) - int(m * 30.6001)

        return datetime(m_year, m_month, m_day, hour, minute, second, tzinfo=utc)

class UTC(tzinfo):
    """
    Represents the UTC timezone
    """

    def utcoffset(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return timedelta(0)

class LocalTimezone(tzinfo):
    """
    Represents the computers local timezone
    """

    def __init__(self):
        self.STDOFFSET = timedelta(seconds = -time.timezone)
        if time.daylight:
            self.DSTOFFSET = timedelta(seconds = -time.altzone)
        else:
            self.DSTOFFSET = self.STDOFFSET

        self.DSTDIFF = self.DSTOFFSET - self.STDOFFSET
        tzinfo.__init__(self)

    def utcoffset(self, dt):
        if self._isdst(dt):
            return self.DSTOFFSET
        else:
            return self.STDOFFSET

    def dst(self, dt):
        if self._isdst(dt):
            return self.DSTDIFF
        else:
            return timedelta(0)

    def tzname(self, dt):
        return time.tzname[self._isdst(dt)]

    def _isdst(self, dt):
        tt = (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second, dt.weekday(), 0, -1)
        stamp = time.mktime(tt)
        tt = time.localtime(stamp)
        return tt.tm_isdst > 0

local = LocalTimezone()
utc = UTC()



# By hads <hads@nice.net.nz>
# Released under the MIT license

import os
import sys

from cgi import escape

from base import *

log = logging.getLogger(NAME)

def get_channels(channel_file):
    channels = {}
    channel_file_ok = False
    channel_info = []
    f = open(channel_file)
    for line in f:
        channel_info.append(line.strip())
    f.close()
    for line in channel_info:
        if line == '# CHANNEL_ID|XMLTVID|NAME|ICON|WEBSITE|CHANNEL_NUMBER':
            channel_file_ok = True
    if not channel_file_ok:
        log.critical("Channel file '%s' appears to be out of date.", channel_file)
        return False

    for chan in channel_info:
        chan = chan.strip()
        if chan and chan[0] != '#':
            fields = chan.split('|')
            channel = Channel(fields[0])
            try:
                channel.xmltvid = fields[1]
            except IndexError:
                pass
            try:
                channel.name = escape(fields[2], quote=True)
            except IndexError:
                pass
            try:
                channel.icon = escape(fields[3], quote=True)
            except IndexError:
                pass
            try:
                channel.url = escape(fields[4], quote=True)
            except IndexError:
                pass
            channels[fields[0]] = channel
    return channels

